@@ -70,12 +70,30 @@ class Chain(Sized):
         """
         raise NotImplementedError()
 
-    def get_draws(self, var_name: str) -> numpy.ndarray:
-        """Retrieve all draws of a variable from an MCMC chain."""
+    def get_draws(self, var_name: str, slc: slice = slice(None)) -> numpy.ndarray:
+        """Retrieve draws of a variable from an MCMC chain.
+
+        Parameters
+        ----------
+        var_name : str
+            Name of the variable.
+        slc : slice, optional
+            Optional ``slice`` object to retrieve only a subset of elements.
+            Passing this can be more performant than slicing the returned value.
+        """
         raise NotImplementedError()
 
-    def get_stats(self, stat_name: str) -> numpy.ndarray:
-        """Retrieve all values of a sampler statistic."""
+    def get_stats(self, stat_name: str, slc: slice = slice(None)) -> numpy.ndarray:
+        """Retrieve values of a sampler statistic.
+
+        Parameters
+        ----------
+        stat_name : str
+            Name of the stats variable.
+        slc : slice, optional
+            Optional ``slice`` object to retrieve only a subset of elements.
+            Passing this can be more performant than slicing the returned value.
+        """
         raise NotImplementedError()
 
     def get_draws_at(self, idx: int, var_names: Sequence[str]) -> Dict[str, numpy.ndarray]:
@@ -152,11 +170,13 @@ class Run:
     def observed_data(self) -> Dict[str, numpy.ndarray]:
         return {dv.name: ndarray_to_numpy(dv.value) for dv in self.meta.data if dv.is_observed}
 
-    def to_inferencedata(self, **kwargs) -> InferenceData:
+    def to_inferencedata(self, *, equalize_chain_lengths: bool = True, **kwargs) -> InferenceData:
         """Creates an ArviZ ``InferenceData`` object from this run.
 
         Parameters
         ----------
+        equalize_chain_lengths : bool
+            Whether to truncate all chains to the shortest chain length (default: ``True``).
         **kwargs
             Will be forwarded to ``arviz.from_dict()``.
 
@@ -180,21 +200,40 @@ class Run:
 
         chain_lengths = {c.cid: len(c) for c in chains}
         if len(set(chain_lengths.values())) != 1:
-            _log.warning("Chains vary in length. Lenghts are: %s", chain_lengths)
-
+            msg = f"Chains vary in length. Lenghts are: {chain_lengths}"
+            if not equalize_chain_lengths:
+                msg += (
+                    "\nArviZ does not properly support uneven chain lengths (see ArviZ issue #2094)."
+                    "\nWe'll try to give you an InferenceData, but best case the chain & draw dimensions"
+                    " will be messed-up as {'chain': 1, 'draws': n_chains}."
+                    "\nYou won't be able to save this InferenceData to a file"
+                    " and you should expect many ArviZ functions to choke on it."
+                    "\nSpecify `to_inferencedata(equalize_chain_lengths=True)` to get regular InferenceData."
+                )
+            else:
+                msg += "\nTruncating to the length of the shortest chain."
+            _log.warning(msg)
+        min_clen = None
+        if equalize_chain_lengths:
+            # A minimum chain length is introduced so that all chains have equal length
+            min_clen = min(chain_lengths.values())
         # Aggregate draws and stats, while splitting into warmup/posterior
         warmup_posterior = collections.defaultdict(list)
         warmup_sample_stats = collections.defaultdict(list)
         posterior = collections.defaultdict(list)
         sample_stats = collections.defaultdict(list)
         for c, chain in enumerate(chains):
-            # Every retrieved array is shortened to the previously determined chain length.
-            # This is needed for database backends which may get inserts inbetween.
-            clen = chain_lengths[chain.cid]
+            # Create a slice to use when fetching the variables
+            if min_clen is None:
+                # Every retrieved array is shortened to the previously determined chain length.
+                # Needed for backends which may get inserts inbetween our get_draws/get_stats calls.
+                slc = slice(0, chain_lengths[chain.cid])
+            else:
+                slc = slice(0, min_clen)
 
             # Obtain a mask by which draws can be split into warmup/posterior
             if "tune" in chain.sample_stats:
-                tune = chain.get_stats("tune")[:clen].astype(bool)
+                tune = chain.get_stats("tune", slc).astype(bool)
             else:
                 if c == 0:
                     _log.warning(
@@ -204,12 +243,12 @@ class Run:
 
             # Split all variables draws into warmup/posterior
             for var in variables:
-                draws = chain.get_draws(var.name)[:clen]
+                draws = chain.get_draws(var.name, slc)
                 warmup_posterior[var.name].append(draws[tune])
                 posterior[var.name].append(draws[~tune])
             # Same for sample stats
             for svar in self.meta.sample_stats:
-                stats = chain.get_stats(svar.name)[:clen]
+                stats = chain.get_stats(svar.name, slc)
                 warmup_sample_stats[svar.name].append(stats[tune])
                 sample_stats[svar.name].append(stats[~tune])
 
